@@ -49,6 +49,12 @@ impl<Tz: chrono::TimeZone> IntoTimestamp for chrono::DateTime<Tz> {
     }
 }
 
+impl IntoTimestamp for i64 {
+    fn into_timestamp(self) -> i64 {
+        self
+    }
+}
+
 /// The custom art assets to be used in the user's profile when the activity
 /// is set. These assets need to be already uploaded to Discord in the application's
 /// developer settings.
@@ -107,8 +113,10 @@ impl Assets {
 /// [API docs](https://discord.com/developers/docs/game-sdk/activities#data-models-activitytimestamps-struct)
 #[derive(Default, Clone, Debug, Serialize, Deserialize)]
 pub struct Timestamps {
-    pub start: i64,
-    pub end: i64,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub start: Option<i64>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub end: Option<i64>,
 }
 
 #[derive(
@@ -145,9 +153,9 @@ pub enum ActivityActionKind {
 }
 
 #[derive(Deserialize, Debug)]
+#[cfg_attr(test, derive(Serialize))]
 pub struct ActivityInvite {
     /// The user that invited the current user to the activity
-    #[serde(deserialize_with = "crate::user::de_user")]
     pub user: crate::user::User,
     /// The activity the invite is for
     pub activity: InviteActivity,
@@ -216,14 +224,12 @@ pub struct Activity {
 }
 
 #[derive(Debug, Deserialize)]
+#[cfg_attr(test, derive(Serialize))]
 pub struct InviteActivity {
     /// The unique identifier for the activity
     pub session_id: String,
     /// The timestamp the activity was created
-    #[serde(
-        skip_serializing,
-        deserialize_with = "crate::util::timestamp::deserialize_opt"
-    )]
+    #[serde(skip_serializing, with = "crate::util::datetime_opt")]
     pub created_at: Option<chrono::DateTime<chrono::Utc>>,
     /// The usual activity data
     #[serde(flatten)]
@@ -287,6 +293,16 @@ impl ActivityBuilder {
         Self::default()
     }
 
+    #[cfg(test)]
+    pub fn with_pid(pid: u32) -> Self {
+        Self {
+            inner: ActivityArgs {
+                pid,
+                activity: None,
+            },
+        }
+    }
+
     /// The user's currenty party status, eg. "Playing Solo".
     ///
     /// Limited to 128 bytes.
@@ -325,39 +341,99 @@ impl ActivityBuilder {
         self
     }
 
+    /// Set the start timestamp for the activity. If only the start is set, Discord will display `XX:XX elapsed`
+    pub fn start_timestamp(mut self, timestamp: impl IntoTimestamp) -> Self {
+        match &mut self.inner.activity {
+            // Modify an existing activity and add a start timestamp
+            Some(activity) => {
+                match &mut activity.timestamps {
+                    // Add a start timestamp to an existing timestamp object
+                    Some(timestamps) => {
+                        timestamps.start = Some(timestamp.into_timestamp());
+                    }
+
+                    // Create a new timestamp object and set its start
+                    None => {
+                        activity.timestamps = Some(Timestamps {
+                            start: Some(timestamp.into_timestamp()),
+                            end: None,
+                        });
+                    }
+                }
+            }
+
+            // Init a new activity with only a start timestamp
+            None => {
+                self.inner.activity = Some(Activity {
+                    timestamps: Some(Timestamps {
+                        start: Some(timestamp.into_timestamp()),
+                        end: None,
+                    }),
+                    ..Default::default()
+                });
+            }
+        }
+
+        self
+    }
+
+    /// Set the end timestamp for the activity. If only the end is set, Discord will display `XX:XX left`
+    pub fn end_timestamp(mut self, timestamp: impl IntoTimestamp) -> Self {
+        match &mut self.inner.activity {
+            // Modify an existing activity and add a start timestamp
+            Some(activity) => {
+                match &mut activity.timestamps {
+                    // Add an end timestamp to an existing timestamp object
+                    // Only done if the end is after the start
+                    Some(timestamps) => {
+                        let timestamp = timestamp.into_timestamp();
+                        let start = timestamps.start.unwrap_or(0);
+
+                        if start > timestamp {
+                            tracing::warn!(
+                                "End timestamp must be greater than the start timestamp"
+                            );
+                        } else {
+                            timestamps.end = Some(timestamp.into_timestamp());
+                        }
+                    }
+
+                    // Create a new timestamp object and set its end
+                    None => {
+                        activity.timestamps = Some(Timestamps {
+                            start: None,
+                            end: Some(timestamp.into_timestamp()),
+                        });
+                    }
+                }
+            }
+
+            // Init a new activity with only an end timestamp
+            None => {
+                self.inner.activity = Some(Activity {
+                    timestamps: Some(Timestamps {
+                        start: None,
+                        end: Some(timestamp.into_timestamp()),
+                    }),
+                    ..Default::default()
+                });
+            }
+        }
+
+        self
+    }
+
     /// The start and end of a "game" or "session".
     pub fn timestamps(
         mut self,
         start: Option<impl IntoTimestamp>,
         end: Option<impl IntoTimestamp>,
     ) -> Self {
-        let start = start.map(IntoTimestamp::into_timestamp);
-        let end = end.map(IntoTimestamp::into_timestamp);
-
-        let timestamps = match (start, end) {
-            (Some(st), Some(et)) => {
-                if st >= et {
-                    tracing::warn!("End timestamp must be greater than the start timestamp");
-                    return self;
-                }
-
-                Some(Timestamps { start: st, end: et })
-            }
-            (None, None) => return self,
-            _ => {
-                tracing::warn!("Both start and end timestamp must be set");
-                return self;
-            }
-        };
-
-        match &mut self.inner.activity {
-            Some(activity) => activity.timestamps = timestamps,
-            None => {
-                self.inner.activity = Some(Activity {
-                    timestamps,
-                    ..Default::default()
-                });
-            }
+        if let Some(st) = start {
+            self = self.start_timestamp(st);
+        }
+        if let Some(et) = end {
+            self = self.end_timestamp(et);
         }
 
         self
@@ -534,7 +610,7 @@ impl crate::Discord {
     /// Accepts the invite to another user's activity.
     ///
     /// [API docs](https://discord.com/developers/docs/game-sdk/activities#acceptinvite)
-    pub async fn accept_invite(&self, invite: &ActivityInvite) -> Result<(), Error> {
+    pub async fn accept_invite(&self, invite: &impl AsRef<ActivityInvite>) -> Result<(), Error> {
         #[derive(Serialize)]
         struct Accept<'stack> {
             user_id: UserId,
@@ -545,6 +621,8 @@ impl crate::Discord {
             message_id: crate::types::MessageId,
         }
 
+        let invite = invite.as_ref();
+
         let rx = self.send_rpc(
             CommandKind::AcceptActivityInvite,
             Accept {
@@ -615,13 +693,54 @@ impl crate::Discord {
 /// so we just truncate them manually client side to avoid sending more data
 #[inline]
 fn truncate(text: Option<impl Into<String>>, name: &str) -> Option<String> {
-    text.map(|text| {
+    text.and_then(|text| {
         let mut text = text.into();
         if text.len() > 128 {
             tracing::warn!("{} '{}' is too long and will be truncated", name, text);
             text.truncate(128);
         }
 
-        text
+        // Ensure the strings don't have just whitespace, as they are also not
+        // allowed
+        if text.trim().is_empty() {
+            None
+        } else {
+            Some(text)
+        }
     })
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn serde() {
+        let args: ActivityArgs = ActivityBuilder::with_pid(9999)
+            .details("deetz")
+            // This won't be set, as empty state is not allowed
+            .state("       ")
+            .start_timestamp(1628629161811)
+            .end_timestamp(1628629327961)
+            .party(
+                "parrrrty",
+                std::num::NonZeroU32::new(1),
+                std::num::NonZeroU32::new(2),
+                PartyPrivacy::Private,
+            )
+            .secrets(Secrets {
+                join: Some("sekret".to_owned()),
+                ..Default::default()
+            })
+            .into();
+
+        let cmd = crate::proto::Rpc {
+            cmd: CommandKind::SetActivity,
+            nonce: 2.to_string(),
+            evt: None,
+            args: Some(args),
+        };
+
+        insta::assert_json_snapshot!(cmd);
+    }
+}
